from lila.core.request import Request
from lila.core.responses import JSONResponse
from pydantic import BaseModel, ValidationError


class Controller:
    async def parse_request(
        self, request: Request, schema: BaseModel, response: JSONResponse = None
    ):
        try:
            body = await request.json()
            return schema(**body)
        except ValidationError as e:
            if response:
                return response(
                    status_code=400,
                    content={"error": True, "details": e.errors()},
                )
            return False
        except Exception as e:
            if response:
                return response(
                    status_code=400,
                    content={"error": True, "details": e.errors()},
                )
            return False



import psutil
import os
import json
from app.helpers.helpers import lang
from lila.core.responses import  RedirectResponse, JSONResponse
from lila.core.request import Request
from lila.core.routing import Router
from lila.core.session import Session
from app.connections import connection
from argon2 import PasswordHasher
from functools import wraps
from lila.core.responses import convert_to_serializable
from lila.core.templates import render
from app.config import PATH_LOG_BASE_DIR as PATH_LOG_BASE_DIR_ADMIN
 
connection = connection
ph = PasswordHasher()


def admin_required(func):
    """Middleware to ensure the user is authenticated as an admin."""

    @wraps(func)
    async def wrapper(request: Request, *args, **kwargs):
        session_data = Session.unsign(key="auth_admin", request=request)
        if not session_data:
            return RedirectResponse(url="/admin/login")
        if session_data["id"] is None:
            return RedirectResponse(url="/admin/login")
        return await func(request, *args, **kwargs)

    return wrapper


def admin_metrics():
    lila_memory, lila_cpu_usage = get_lila_memory_usage()
    system_used_memory, system_total_memory, cpu_usage = get_system_memory_usage()
    metrics = {
        "lila_memory": lila_memory,
        "lila_cpu_usage": lila_cpu_usage,
        "system_used_memory": system_used_memory,
        "system_total_memory": system_total_memory,
        "cpu_usage": cpu_usage,
    }
    return JSONResponse(metrics)


def get_lila_memory_usage() -> tuple:
    """Get memory and CPU usage of the Lila Framework process."""
    process = psutil.Process(os.getpid())
    memory_usage = process.memory_info().rss / (1024 * 1024)
    cpu_usage = process.cpu_percent()
    return memory_usage, cpu_usage


def get_system_memory_usage() -> tuple:
    """Get system memory and CPU usage."""
    memory_info = psutil.virtual_memory()
    used_memory = memory_info.used / (1024 * 1024)
    total_memory = memory_info.total / (1024 * 1024)
    cpu_usage = psutil.cpu_percent()
    return used_memory, total_memory, cpu_usage


def authenticate(username: str, password: str) -> dict:
    """Authenticate an admin user.

    Args:
        username (str): The username of the admin.
        password (str): The password of the admin.

    Returns:
        dict: The admin data if authentication is successful, otherwise None.
    """
    query = "SELECT id, username, password FROM admins WHERE username = :username AND active = 1 LIMIT 1"
    params = {"username": username}
    admin = connection.query(query=query, params=params, return_row=True)
    if not admin or not admin.get("password"):
        return None

    try:
        if ph.verify(admin["password"], password):
            admin["password"] = None
            return admin
    except Exception as e:
        print(f"Error verifying password: {e}")
        return None

    return None


async def admin_login(request: Request):
    """Handle admin login requests."""
    if request.method == "POST":
        try:
            form_data = await request.json()
            username = form_data.get("user",None)
            password = form_data.get("password",None)
            admin = authenticate(username=username, password=password)
            if admin:
                response = JSONResponse(
                    {"success": True, "redirect": "/admin"}, serialize=False
                )
                admin_val = {"id": admin["id"]}
                Session.setSession(
                    new_val=admin_val, response=response, name_cookie="auth_admin"
                )
                return response
            return JSONResponse(
                {"success": False, "message": "Invalid credentials"},
                status_code=401,
            )
        except Exception as e:
            print(e)
            return JSONResponse({"success": False, "message": "Error"}, status_code=500)

    return render(request=request, template="admin/login")


async def admin_dashboard(request: Request, menu: str = "") -> str:
    """Generate the HTML for the admin dashboard."""
    lila_memory, lila_cpu_usage = get_lila_memory_usage()
    system_used_memory, system_total_memory, cpu_usage = get_system_memory_usage()

    logs = {}
    PATH_LOG_BASE_DIR = PATH_LOG_BASE_DIR_ADMIN
    logs_html = ""
    if os.path.exists(PATH_LOG_BASE_DIR):
        for log_folder in os.listdir(PATH_LOG_BASE_DIR):
            log_folder_path = os.path.join(PATH_LOG_BASE_DIR, log_folder)
            if os.path.isdir(log_folder_path):
                logs[log_folder] = {}
                for log_file in os.listdir(log_folder_path):
                    if log_file.endswith(".log"):
                        with open(os.path.join(log_folder_path, log_file), "r") as f:
                            logs[log_folder][log_file] = f.readlines()

        logs_html = f"""
        <details class="bg-gray-50 dark:bg-gray-700 rounded-lg p-2 border border-gray-200 dark:border-gray-600">
            <summary class="logs-summary font-bold cursor-pointer p-2 bg-gray-200 dark:bg-gray-600 rounded-md text-gray-800 dark:text-gray-200 flex items-center space-x-2">
                📁 Logs
            </summary>
            <ul class="logs-list list-none pl-0 mt-2 space-y-2">
        """
        for log_folder, log_files in logs.items():
            logs_html += f"""
            <li class="log-item">
                <details class="log-folder bg-white dark:bg-gray-800 rounded-lg shadow-sm">
                    <summary class="log-folder-summary font-medium cursor-pointer p-2 border-b dark:border-gray-700 flex items-center space-x-2">
                        📁 {log_folder}
                    </summary>
                    <ul class="log-files-list list-none pl-0 mt-2 space-y-2">
            """
            for log_file, log_lines in log_files.items():
                logs_html += f"""
                <li class="log-file-item">
                    <details class="log-file bg-white dark:bg-gray-800 rounded-lg border border-gray-200 dark:border-gray-700 p-2">
                        <summary class="log-summary font-medium cursor-pointer flex items-center space-x-2">
                            📄 {log_file}
                        </summary>
                        <pre class="log-content mt-2 p-2 border border-gray-300 dark:border-gray-600 rounded-md max-h-96 overflow-y-auto font-mono text-sm whitespace-pre-wrap break-words bg-black text-lime-400">{"".join(log_lines)}</pre>
                    </details>
                </li>
                """
            logs_html += """
                    </ul>
                </details>
            </li>
            """
        logs_html += """
            </ul>
        </details>
        """

    menu_html = menu

    context = {
        "lila_memory": lila_memory,
        "lila_cpu_usage": lila_cpu_usage,
        "system_used_memory": system_used_memory,
        "system_total_memory": system_total_memory,
        "cpu_usage": cpu_usage,
        "logs_html": logs_html,
        "menu": menu_html,
    }

    return render(request=request,template="admin/dashboard",context=context)


def menu(models: list = []) -> str:
    """Generate the admin menu."""

    m = ""
    
    for model in models:
        model_name = model.__name__.lower()
        model_plural = f"{model_name}s"
        # m += f"<li><a href='/admin/{model_plural}' class='secondary mt-2'>{model_name.capitalize()}</a></li>"
        m += f"<li><a href='/admin/{model_plural}' class='block px-4 py-2 hover:bg-gray-100 dark:hover:bg-gray-700 rounded-md transition-colors'>{model_name.capitalize()}</a></li>"

    return f"""
    <header class="bg-white dark:bg-gray-800 shadow-md p-4">
        <nav class="container mx-auto flex items-center justify-between">
            <div class="flex items-center space-x-4">
                <a href="/admin/" class="text-xl font-bold text-gray-900 dark:text-gray-100 hover:text-blue-600 dark:hover:text-blue-400 transition-colors">Admin Dashboard</a>
            </div>
            
            <div class="relative">
                <details class="group">
                    <summary class="flex items-center space-x-2 px-4 py-2 rounded-md cursor-pointer bg-gray-200 dark:bg-gray-700 text-gray-800 dark:text-gray-200 hover:bg-gray-300 dark:hover:bg-gray-600 transition-colors">
                        <span>Menu</span>
                        <svg class="w-4 h-4 transform group-open:rotate-180 transition-transform" fill="currentColor" viewBox="0 0 20 20" xmlns="http://www.w3.org/2000/svg">
                            <path fill-rule="evenodd" d="M5.293 7.293a1 1 0 011.414 0L10 10.586l3.293-3.293a1 1 0 111.414 1.414l-4 4a1 1 0 01-1.414 0l-4-4a1 1 0 010-1.414z" clip-rule="evenodd" />
                        </svg>
                    </summary>
                    <ul class="absolute right-0 mt-2 w-48 bg-white dark:bg-gray-800 border border-gray-200 dark:border-gray-700 rounded-md shadow-lg z-10 p-2 space-y-1">
                        {m}
                        <li>
                            <a href="/admin/logout" class="block px-4 py-2 text-red-500 hover:bg-gray-100 dark:hover:bg-gray-700 rounded-md transition-colors">Logout</a>
                        </li>
                    </ul>
                </details>
            </div>
        </nav>
    </header>
    """
def admin_routes(models: list, router: Router,default_route: str = "admin") -> Router:

    @router.route(path=f"/{default_route}/logout", methods=["GET"])
    async def admin_logout(request: Request):
        """Handle admin logout requests."""
        response = RedirectResponse(url=f"/{default_route}/login")
        response.delete_cookie("auth_admin")
        return response

    @router.route(path=f"/{default_route}/login", methods=["GET", "POST"])
    async def admin_login_route(request: Request):
        """Handle admin login requests."""
        return await admin_login(request=request)

    @router.route(path=f"/{default_route}/metrics", methods=["GET"])
    @admin_required
    async def get_metrics(request: Request):
        return admin_metrics()

    @router.route(path=f"/{default_route}", methods=["GET"])
    @admin_required
    async def admin_route(request: Request):
        menu_html = menu(models=models)
        return await admin_dashboard(request=request, menu=menu_html)

    for model in models:
        model_name = model.__name__.lower()
        model_plural = f"{model_name}s"

        @router.route(path=f"/{default_route}/{model_plural}", methods=["GET"])
        @admin_required
        async def model_list(request: Request, model=model, model_name=model_name):
            items = model.get_all()
            items = convert_to_serializable(items)

            headers = items[0].keys() if items else []
            html_lang = lang(request=request)

            context = {
                "request": request,
                "items_json": json.dumps(items),
                "model_name": model_name,
                "headers": headers,
                "html_lang": html_lang,
                "menu": menu(models=models),
            }

            return render(request=request,template="admin/model_table", context=context)

    return router



import os
import datetime
from datetime import timedelta
import traceback
from lila.core.request import Request
import re
from app.config import PATH_LOG_BASE_DIR

if not os.path.exists(PATH_LOG_BASE_DIR):
    os.makedirs(PATH_LOG_BASE_DIR)


def folder_logs(type):
    now = datetime.datetime.now().strftime("%d-%m-%Y")
    path_ = f"{PATH_LOG_BASE_DIR}/{now}"
    os.makedirs(path_, exist_ok=True)
    return f"{path_}/{type}.log"


class Logger:

    @classmethod
    def _write_log(cls, log_type, message):
        log_file = folder_logs(log_type)
        timestamp = datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S")
        with open(log_file, "a") as file:
            file.write(f"{timestamp} - {log_type.upper()} - {message}\n\n\n")

    @classmethod
    def error(cls, message: str, exception: Exception = None):
        if exception:
            message += f"\n{traceback.format_exc()}"
        cls._write_log("error", message)

    @classmethod
    def warning(cls, message: str):
        cls._write_log("warning", message)

    @classmethod
    def info(cls, message: str):
        cls._write_log("info", message)

    @classmethod
    def log(cls,log_type:str,message:str)->None:
        cls._write_log(log_type,message)

    @classmethod
    async def request(cls, request: Request):
        client_ip = request.client.host
        user_agent = request.headers.get("user-agent", "Unknown")
        os_info = "Unknown"

        os_patterns = {
            "Windows": r"Windows NT|Windows",
            "Linux": r"Linux|X11",
            "Mac OS": r"Macintosh|Mac OS X",
            "iOS": r"iPhone|iPad|iPod",
            "Android": r"Android",
        }

        for os_name, pattern in os_patterns.items():
            if re.search(pattern, user_agent, re.IGNORECASE):
                os_info = os_name
                break
        try:
            body = await request.body()
            body_content = body.decode() if body else ""
            if "password" in body_content:
                body_content["password"] = ""
            if "password_2" in body_content:
                body_content["password_2"] = ""
            if "token" in body_content:
                body_content["token"] = ""
        except Exception as e:
            body_content = ""
        try:
            json_content = await request.json()
            if "password" in json_content:
                json_content["password"] = ""
            if "password_2" in json_content:
                json_content["password_2"] = ""
            if "token" in json_content:
                json_content["token"] = ""

        except Exception as e:
            json_content = {}
        return (
            f"IP: {client_ip}, "
            f"OS: {os_info}, "
            f"URL: {request.url.path}, "
            f"Method: {request.method} | "
            f"Query Params: {dict(request.query_params)} | "
            f"Path Params: {request.path_params} | "
            f"Body: {body_content} | "
            f"JSON: {json_content}"
        )


def delete_old_logs(days: int = 30):
    now = datetime.datetime.now()
    for folder_name in os.listdir(PATH_LOG_BASE_DIR):
        folder_path = os.path.join(PATH_LOG_BASE_DIR, folder_name)
        try:
            folder_date = datetime.datetime.strptime(folder_name, "%d-%m-%Y")
            if now - folder_date > timedelta(days=days):
                for file in os.listdir(folder_path):
                    os.remove(os.path.join(folder_path, file))
                os.rmdir(folder_path)
        except ValueError:
            continue


delete_old_logs()



from sqlalchemy import Table, Column, Integer, String, TIMESTAMP
from sqlalchemy.orm import Session
from lila.core.database import Base
from app.connections import connection
from argon2 import PasswordHasher

ph = PasswordHasher()


class User(Base):
    __tablename__ = "users"
    id = Column(Integer, primary_key=True, autoincrement=True)
    name = Column(String(length=50), nullable=False)
    email = Column(String(length=50), unique=True)
    password = Column(String(length=150), nullable=False)
    token = Column(String(length=150), nullable=False)
    active = Column(Integer, nullable=False, default=1)
    created_at = Column(TIMESTAMP)

    # English : Example of how to use SQLAlchemy to make queries to the database
    # Español : Ejemplo de como poder utilizar SQLAlchemy para hacer consultas a la base de datos
    def get_all(select: str = "id,email,name", limit: int = 1000) -> list:
        query = f"SELECT {select}  FROM users WHERE active =1  LIMIT {limit}"
        result = connection.query(query=query,return_rows=True)
        return result 

    # English : Example of how to use SQLAlchemy to make queries to the database
    # Español : Ejemplo de como poder utilizar SQLAlchemy para hacer consultas a la base de datos
    def get_by_id(id: int, select="id,email,name") -> dict:
        query = f"SELECT {select}  FROM users WHERE id = :id AND active = 1 LIMIT 1"
        params = {"id": id}
        row = connection.query(query=query, params=params,return_row=True)
        return row

    # English: Example using ORM abstraction in SQLAlchemy
    # Español : Ejemplo usando abstracción de ORM en SQLAlchemy
    @classmethod
    def get_all_orm(cls, db: Session, limit: int = 1000):
        result = db.query(cls).filter(cls.active == 1).limit(limit).all()
        return result


# English : Example of how to use the class to make queries to the database
# Español : Ejemplo de como usar la clase para realizar consultas a la base de datos
# users = User.get_all()
# user = User.get_by_id(1)



from abc import ABC, abstractmethod
from starlette.middleware.base import BaseHTTPMiddleware
from lila.core.responses import HTMLResponse, JSONResponse
from lila.core.request import Request
from lila.core.logger import Logger
from datetime import datetime
import json
import os
import traceback
import sys
from typing import Any
from app.config import PATH_SECURITY
from app.config import SENSITIVE_PATHS


class BaseSecurityMiddleware(BaseHTTPMiddleware, ABC):
    def __init__(self, app, config_file: str, default_value: Any):
        super().__init__(app)
        self.config_file = config_file
        self.default_value = default_value
        self.data = self.load_data()

    def load_data(self):
        try:
            if not os.path.exists(self.config_file):
                os.makedirs(os.path.dirname(self.config_file), exist_ok=True)
                with open(self.config_file, "w") as file:
                    json.dump(self.default_value, file, indent=4)
                return self.default_value

            with open(self.config_file, "r") as file:
                content = file.read().strip()
                if not content:
                    with open(self.config_file, "w") as file:
                        json.dump(self.default_value, file, indent=4)
                    return self.default_value

                try:
                    return json.loads(content)
                except json.JSONDecodeError:
                    with open(self.config_file, "w") as file:
                        json.dump(self.default_value, file, indent=4)
                    return self.default_value

        except Exception as e:
            Logger.error(f"Error loading {self.config_file}: {str(e)}")
            return self.default_value

    def save_data(self, data):
        try:
            os.makedirs(os.path.dirname(self.config_file), exist_ok=True)
            with open(self.config_file, "w") as file:
                json.dump(data, file, indent=4)
        except Exception as e:
            Logger.error(f"Error saving {self.config_file}: {str(e)}")

    async def is_blocked(self, key: str, request: Request) -> bool:
        if key in self.data:
            expiration_time = datetime.fromisoformat(self.data[key]["expiration_time"])
            if datetime.now() < expiration_time:
                req = await Logger.request(request=request)
                Logger.warning(f"Blocked by {self.__class__.__name__}: {key} \n {req}")
                return True
        return False

    @abstractmethod
    async def check_request(self, request: Request) -> bool:
        pass

    @abstractmethod
    async def handle_blocked_request(self, request: Request) -> HTMLResponse:
        pass

    async def dispatch(self, request, call_next):
        if await self.check_request(request):
            return await self.handle_blocked_request(request)

        response = await call_next(request)
        return response


class ErrorHandlerMiddleware(BaseHTTPMiddleware):
    async def dispatch(self, request, call_next):
        try:
            response = await call_next(request)
            return response
        except Exception as e:
            exc_type, exc_value, exc_tb = sys.exc_info()

            if exc_tb:
                tb = exc_tb
                while tb.tb_next:
                    tb = tb.tb_next

                frame = tb.tb_frame
                filename = frame.f_code.co_filename
                lineno = tb.tb_lineno
                function = frame.f_code.co_name

                error_info = {
                    "type": exc_type.__name__,
                    "message": str(e),
                    "file": filename,
                    "line": lineno,
                    "function": function,
                    "traceback": traceback.format_exc(),
                }

                Logger.error(
                    f"Error: {error_info['type']} in {error_info['file']}:{error_info['line']}"
                )
                Logger.error(f"Function: {error_info['function']}")
                Logger.error(f"Message: {error_info['message']}")
                Logger.error(f"Traceback:\n{error_info['traceback']}")

            return JSONResponse(
                {"error": "Internal Server Error", "success": False}, status_code=500
            )


class IPBlockingMiddleware(BaseSecurityMiddleware):
    def __init__(self, app, blocked_ips_file=f"{PATH_SECURITY}/blocked_ips.json"):
        super().__init__(app, blocked_ips_file, default_value={})

    async def check_request(self, request: Request) -> bool:
        client_ip = request.client.host
        return await self.is_blocked(client_ip, request)

    async def handle_blocked_request(self, request: Request) -> HTMLResponse:
        return HTMLResponse(
            content="<h1>Access Denied</h1><p>Your IP has been temporarily blocked.</p>",
            status_code=403,
        )


class URLBlockingMiddleware(BaseSecurityMiddleware):
    def __init__(self, app, blocked_urls_file=f"{PATH_SECURITY}blocked_urls.json"):
        super().__init__(app, blocked_urls_file, default_value={})

    async def check_request(self, request: Request) -> bool:
        url_path = request.url.path
        return await self.is_blocked(url_path, request)

    async def handle_blocked_request(self, request: Request) -> HTMLResponse:
        return HTMLResponse(
            content="<h1>Access Denied</h1><p>This URL has been temporarily blocked.</p>",
            status_code=403,
        )


class MaliciousExtensionMiddleware(BaseHTTPMiddleware):
    def __init__(self, app, malicious_extensions=None, block_duration_hours=6):
        super().__init__(app)
        self.malicious_extensions = malicious_extensions or [
            ".php",
            ".asp",
            ".jsp",
            ".aspx",
        ]
        self.block_duration_hours = block_duration_hours

    async def dispatch(self, request, call_next):
        url_path = request.url.path.lower()

        if any(ext in url_path for ext in self.malicious_extensions):
            client_ip = request.client.host
            Logger.warning(f"Malicious extension detected from {client_ip}: {url_path}")
            return HTMLResponse(
                content="<h1>Access Denied</h1><p>Malicious URL detected.</p>",
                status_code=403,
            )

        response = await call_next(request)
        return response


class SensitivePathMiddleware(BaseHTTPMiddleware):
    def __init__(
        self, app, sensitive_paths_file=f"{PATH_SECURITY}sensitive_paths.json"
    ):
        super().__init__(app)
        self.sensitive_paths = self.load_sensitive_paths(sensitive_paths_file)

    def load_sensitive_paths(self, file_path):
        default_paths = SENSITIVE_PATHS
        try:
            if not os.path.exists(file_path):
                os.makedirs(os.path.dirname(file_path), exist_ok=True)
                with open(file_path, "w") as file:
                    json.dump(default_paths, file, indent=4)
                return default_paths

            with open(file_path, "r") as file:
                content = file.read().strip()
                if not content:
                    with open(file_path, "w") as file:
                        json.dump(default_paths, file, indent=4)
                    return default_paths

                try:
                    return json.loads(content)
                except json.JSONDecodeError:
                    with open(file_path, "w") as file:
                        json.dump(default_paths, file, indent=4)
                    return default_paths

        except Exception as e:
            Logger.error(f"Error loading sensitive paths: {str(e)}")
            return default_paths

    async def dispatch(self, request, call_next):
        url_path = request.url.path.lower()
        body = await request.body()
        body_str = str(body).lower()

        if any(path in url_path or path in body_str for path in self.sensitive_paths):
            client_ip = request.client.host
            Logger.warning(f"Sensitive path detected from {client_ip}: {url_path}")
            return HTMLResponse(
                content="<h1>Access Denied</h1><p>Sensitive path detected.</p>",
                status_code=403,
            )

        response = await call_next(request)
        return response


class LoggingMiddleware(BaseHTTPMiddleware):
    def __init__(self, app, exclude_extensions=None, exclude_paths=None):
        super().__init__(app)
        self.exclude_extensions = exclude_extensions or {
            ".js",
            ".css",
            ".jpg",
            ".jpeg",
            ".png",
            ".gif",
            ".ico",
            ".svg",
            ".woff",
            ".woff2",
            ".ttf",
            ".eot",
        }
        self.exclude_paths = exclude_paths or {
            "public",
            "static",
            "assets",
            "favicon.ico",
        }

    async def should_log(self, path: str) -> bool:
        path_lower = path.lower()
        if any(path_lower.endswith(ext) for ext in self.exclude_extensions):
            return False
        if any(excluded in path_lower for excluded in self.exclude_paths):
            return False
        return True

    async def dispatch(self, request, call_next):
        if await self.should_log(request.url.path):
            Logger.info(await Logger.request(request=request))

        response = await call_next(request)
        return response



import typer
import asyncio
import os
import sys
import shlex
import shutil
import sys

app = typer.Typer()

# Project root path / Ruta raíz del proyecto
project_root = os.path.abspath(os.path.join(os.path.dirname(__file__), ".."))


async def run_command(command, cwd=None):
    """
    Run a shell command and show output in real time.
    Ejecuta un comando de terminal mostrando la salida en vivo.
    """
    process = await asyncio.create_subprocess_shell(
        command,
        cwd=cwd,
        stdout=asyncio.subprocess.PIPE,
        stderr=asyncio.subprocess.STDOUT
    )

    # Read and print output line by line / Leer e imprimir línea por línea
    while True:
        line = await process.stdout.readline()
        if not line:
            break
        sys.stdout.write(line.decode())
        sys.stdout.flush()

    return await process.wait()


async def create_react_app(name: str):
    """
    Create a new React app in the project root with Vite.
    Crea una nueva app React en la raíz del proyecto con Vite.
    """
    file_path = os.path.join(project_root, "main.py")

    # Check if main.py exists / Verificar si main.py existe
    if not os.path.exists(file_path):
        print("❌ You must first run lila-init in the terminal to create the lila application.\n"
              "❌ Primero debes ejecutar lila-init en la terminal para crear la aplicación lila.")
        return

    # Path where the React app will be created / Ruta donde se creará la app React
    project_dir = os.path.join(project_root, name)

    # Step 1: Create React app with Vite / Paso 1: Crear app React con Vite
    print(f"⚡ Creating React app '{name}' in {project_dir}...\n"
          f"⚡ Creando la app React '{name}' en {project_dir}...\n")
    #await run_command("npm install -g create-vite")
    #exit_code = await run_command(f"npm create vite@latest {shlex.quote(project_dir)} -- --template react")
    exit_code = await run_command(
        f"npx --yes create-vite {shlex.quote(name)} --template react"
        )


    if exit_code != 0:
        print("❌ Error creating React app / Error creando la app React.")
        return

    # Step 2: Install dependencies / Paso 2: Instalar dependencias
    print("\n📦 Installing dependencies...\n"
          "📦 Instalando dependencias...\n")
    exit_code = await run_command("npm install", cwd=project_dir)
    if exit_code != 0:
        print("❌ Error installing dependencies / Error instalando dependencias.")
        return

    # Step 3: Configure vite.config.js / Paso 3: Configurar vite.config.js
    vite_config_path = os.path.join(project_dir, "vite.config.js")
    with open(vite_config_path, "w") as f:
        f.write(
            """import { defineConfig } from 'vite'
import react from '@vitejs/plugin-react'
import { resolve } from 'path'

export default defineConfig({
  plugins: [react()],
  build: {
    outDir: resolve(__dirname, '../templates/html/react'), 
    emptyOutDir: true,
    rollupOptions: {
      output: {
        entryFileNames: `assets/[name].js`,
        chunkFileNames: `assets/[name].js`,
        assetFileNames: `assets/[name].[ext]`
      }
    }
  }
})

"""
        )

    # Step 4: Update CORS config in main.py / Paso 4: Actualizar configuración CORS en main.py
    print("\n🔧 Updating CORS configuration in main.py...\n"
          "🔧 Actualizando configuración CORS en main.py...\n")
    marker = "react_marker"
    replace_text = """
#English: for development react
#Espanol: para desarrollo react
cors={
    "origin": ["http://localhost:5173"],
    "allow_credentials" : True,
    "allow_methods":["*"],
    "allow_headers": ["*"]
}
 
    """

    with open(file_path, "r",encoding="utf-8") as file:
        content = file.read()

    if marker in content:
        new_content = content.replace(marker, f"{marker}\n{replace_text}")
        with open(file_path, "w",encoding="utf-8") as file:
            file.write(new_content)
        print("✅ CORS configuration inserted / Configuración CORS insertada")
    else:
        print("ℹ️ Marker not found / Marcador no encontrado")

    src_logo = os.path.join(project_root, "static/img/lila.png")
    dest_logo = os.path.join(project_dir, "src/assets/lila.png")
    os.makedirs(os.path.dirname(dest_logo), exist_ok=True)
    shutil.copyfile(src_logo, dest_logo)

    css_files = ["index.css", "App.css"]
    for css_file in css_files:
        path = os.path.join(project_dir, "src", css_file)
        if os.path.exists(path):
            os.remove(path)

    app_jsx_path = os.path.join(project_dir, "src", "App.jsx")
    react_app_default_lila = """
    import { useState } from 'react'
    import lilaLogo from './assets/lila.png'

function App() {
  const [count, setCount] = useState(0)

  return (
    <>
      
      <div className="flex flex-col items-center justify-center min-h-screen bg-gray-50 dark:bg-gray-900 p-4">
        <div className="flex space-x-4 mb-6">
          <img src={lilaLogo} alt="Lila Logo" className="h-20 w-20" />
          <h1 className="text-4xl font-bold text-gray-800 dark:text-gray-100">React + Lila</h1>
        </div>

        <h5 className="text-xl text-center text-gray-800 dark:text-gray-300 mb-4">
          English: All the power of React with Lila Framework, mixing the best
          of both worlds, rendering the react index with lila and using the
          interactivity of react, automatic middlewares, security, admin panel,
          speed, performance, automatically generated rest crud api, and much
          more.
          <br />
          <br />
          Español: Todo el poder React con Lila Framework, pudiendo mezclar lo
          mejor de ambos mundos, renderizando el index de react con lila y
          usando la interactividad de react, middlewares automaticos,seguridad,
          panel de admin, velocidad, rendimiento, api rest crud generada
          automaticamente, y mucho mas.
        </h5>

        <p className="text-center text-gray-700 dark:text-gray-300 mb-4">
        English: You can use react-router or whatever you want to handle routes with react,
        modifying app/routes/routes.py to render the react index directly like this.<br />
        Español: Puedes utilizar react-router o lo que quieras para manejar las rutas con react,
        modificando app/routes/routes.py para que renderice el index de react directamente de esta forma.
        <br />
       <code className="block bg-gray-100 dark:bg-gray-800 p-4 rounded-lg">
router.mount(path="/assets",directory="templates/html/react/assets",name="react-assets")
@router.route(path="/&#123;path:path&#125;", methods=["GET"])
<br />
async def home(request: Request):
    <br />
    &nbsp;&nbsp;response = render(
    <br />
    &nbsp;&nbsp;&nbsp;&nbsp;request=request, template="react/index"
    <br />
    &nbsp;&nbsp;)
    <br />
    &nbsp;&nbsp;return response
    </code>
        </p>

        <p className="text-center text-gray-700 dark:text-gray-300 mb-4">
            English: Remember that in <a href="/docs" target="_blank">docs</a> you can find the automatic documentation generated by Lila in app/routes/api.py.<br />

            Español: Recuerda que en  <a href="/docs" target="_blank">docs</a> puedes encontrar la documentación
            automatica generada por lila en app/routes/api.py .
        </p>

        <p className="text-center text-gray-700 dark:text-gray-300 mb-4">
          English: You can use Lila to have SEO recognizable by Google Bots, AI with hydration,
            and a caching system. Read the documentation with the example at 
            <a href="https://seip25.github.io/Lila/documentation.html#react-seo" target="_blank" className="text-blue-500 hover:underline">
            https://seip25.github.io/Lila/documentation.html#react-seo 
             </a>
            .<br />
          
          Español: Puedes usar Lila para tener SEO reconocible por Bots de Google,IA con hidratación
            y sistema de caching , lee la documentación con el ejemplo en
             <a href="https://seip25.github.io/Lila/esp/documentation.html#react-seo" target="_blank" className="text-blue-500 hover:underline">
            https://seip25.github.io/Lila/esp/documentation.html#react-seo 
             </a>.
        
        </p>

        <div className="bg-white dark:bg-gray-800 p-6 rounded-xl shadow-md mb-6">
        <div className='flex items-center justify-center mb-4'>
            <button
            className="px-4 py-2 bg-blue-600 text-white rounded hover:bg-blue-700 transition "
            onClick={() => setCount(count + 1)}
          >
            Count is {count}
          </button>
          </div>
          <p className="mt-2 text-gray-600 dark:text-gray-300 text-sm">
            Click to test interactivity / Haz click para probar la interactividad
          </p>
        </div>

        <p className="text-gray-500 dark:text-gray-400 text-sm">
          🔗 Documentation: <a href="https://seip25.github.io/Lila/documentation.html" target="_blank" className="text-blue-500 hover:underline">https://seip25.github.io/Lila</a>
        <br/>
        🔗 Documentación: <a href="https://seip25.github.io/Lila/esp/documentation.html" target="_blank" className="text-blue-500 hover:underline">https://seip25.github.io/Lila</a>
        </p>
      </div>
    </>
  )
}

export default App
"""
    with open(app_jsx_path, "w",encoding="utf-8") as f:
        f.write(react_app_default_lila)

    main_js_path = os.path.join(project_dir, "src", "main.jsx")
    react_main_default_lila = """
    import { StrictMode } from 'react'
    import { createRoot } from 'react-dom/client'
    
    import App from './App.jsx'

    createRoot(document.getElementById('root')).render(
    <StrictMode>
        <App />
    </StrictMode>,
    )
    """
    with open(main_js_path, "w", encoding="utf-8") as f:
        f.write(react_main_default_lila)

    index_html_path = os.path.join(project_dir, "index.html")
    react_html_default_lila = """
            <!doctype html>
                <html lang="en">
                <head>
                <meta charset="UTF-8" />
                <link rel="icon" type="image/svg+xml" href="/vite.svg" />
                <meta name="viewport" content="width=device-width, initial-scale=1.0" />
                <title>Vite + React</title>
                <script src="https://cdn.tailwindcss.com"></script>
                </head>
                <body>
                <div id="root">{% include "react/cache/index.html" ignore missing %} </div>
                <script type="module" src="/src/main.jsx"></script>
                </body>
            </html>
    """
    with open(index_html_path, "r+", encoding="utf-8") as f:
        f.write(react_html_default_lila)


    file_routes= os.path.join(project_root, "app", "routes", "routes.py")
    if os.path.exists(file_routes):
        with open(file_routes, "r",encoding="utf-8") as file:
            content = file.read()

        if "marker_react" in content:
            replace_text = """marker_react
# English: Mounting the React app assets and defining the route to render the React index.
# Espanol: Montando los assets de la app React y definiendo la ruta para renderizar el index de React.
router.mount(path="/assets",directory="templates/html/react/assets",name="react-assets")
@router.route(path="/", methods=["GET"])
async def home(request: Request):
  response = render(request=request, template="react/index")
  return response
  """
            new_content = content.replace("marker_react", replace_text)
            with open(file_routes, "w",encoding="utf-8") as file:
                file.write(new_content)
    
    print("\n🏗️ Building the React app...\n"
      "🏗️ Construyendo la aplicación React...\n")
    exit_code = await run_command("npm run build", cwd=project_dir)
    if exit_code != 0:
        print("❌ Error building React app / Error construyendo la app React.")
        return


    print("\n🎉 React app created successfully! / ¡App React creada exitosamente!\n")
    print(f"➡ cd {name}")
    print("➡ npm run dev  (start development server / iniciar servidor de desarrollo)")
    print("➡ Open your browser at http://localhost:5173 / Abre tu navegador en http://localhost:5173")
    print("➡ To build for production, run: npm run build / Para construir para producción, ejecuta: npm run build")
    print("➡ Update your routes in app/routes/routes.py to include the React app for production/ Actualiza tus rutas en app/routes/routes.py para incluir la app React para producción")
    print("""➡ Example/ Ejemplo: 
router.mount(path="/assets",directory="templates/html/react/assets",name="react-assets")
@router.route(path="/{path:path}", methods=["GET"])
async def home(request: Request):
    response = render(
        request=request, template="react/index"
    )  
    return response
    """)
    print("\nYou can now start developing your React app! / ¡Ahora puedes comenzar a desarrollar tu app React!\n")

@app.command()
def create(name: str = 'react'):
    """Create a new React application using Vite / Crear nueva app React usando Vite"""
    asyncio.run(create_react_app(name))


if __name__ == "__main__":
    app()







from lila.core.routing import Router
from lila.core.admin import admin_routes

router = Router()

def Admin(models: list,default_route: str = "admin") -> Router:
    """
    Function to register admin routes for the given models.
    Returns the router with all registered routes.
    """ 
    generated_routes = admin_routes(models=models, router=router,default_route=default_route)
    return generated_routes.get_routes()


routes = router.get_routes() 


import typer
from app.helpers.helpers import generate_token_value
from app.connections import connection
from argon2 import PasswordHasher
import os

ph = PasswordHasher()

app = typer.Typer()

project_root = os.path.abspath(os.path.join(os.path.dirname(__file__), ".."))

@app.command(name="create_admin")
def create_admin(user: str = "admin", password: str = None):
    """
    Create user command.
    """
    success = check_and_create_table()
    if not password:
        password = generate_token_value(2)

    success = create_admin(user, password)
    if not success:
        typer.echo(f"Failed to create admin user '{user}'.")
        raise typer.Exit(code=1)
    main_file=os.path.join(project_root, "main.py")
    marker = "admin_marker"
    replace_text = f"""
from app.routes.admin import Admin
from app.models.user import User
admin_routes=Admin(models=[User])
all_routes = list(itertools.chain(routes, api_routes,admin_routes))
    """
    if os.path.exists(main_file):
         with open(main_file, "r",encoding="utf-8") as file:
            content = file.read()

    if marker in content:
        new_content = content.replace(marker, f"{marker}\n{replace_text}")
        with open(main_file, "w",encoding="utf-8") as file:
            file.write(new_content)
        
    typer.echo(f"Admin user '{user}' created with password '{password}'.")
    typer.echo("Admin command executed.")


def create_admin(username: str, password: str) -> bool:
    """Create a new admin user.

    Args:
        username (str): The username of the admin.
        password (str): The password of the admin.

    Returns:
        bool: True if the admin was created successfully, False otherwise.
    """
    hashed_password = ph.hash(password)
    query = "INSERT INTO admins (username, password, active) VALUES (:username, :password, 1)"
    params = {"username": username, "password": hashed_password}
    result = connection.query(query=query, params=params)
    return result


def check_and_create_table():
    """Check if the 'admins' table exists and create it if it doesn't."""
    db_type = connection.engine.url.drivername

    if db_type == "sqlite":
        query = "SELECT name FROM sqlite_master WHERE type='table' AND name='admins'"
    elif db_type in {"postgresql", "mysql", "mysql+mysqlconnector"}:
        query = """
            SELECT EXISTS (
                SELECT 1
                FROM information_schema.tables
                WHERE table_name = 'admins'
            )
            """
    else:
        raise ValueError(f"Unsupported database type: {db_type}")

    table_exists = connection.query(query=query, return_row=True)
    if db_type == "sqlite":
        table_exists = bool(table_exists)
    elif db_type in {"postgresql", "mysql", "mysql+mysqlconnector"}:
        table_exists = table_exists.get("table_exists", 0) == 1

    if not table_exists:
        if db_type == "sqlite":
            column_definition = "id INTEGER PRIMARY KEY AUTOINCREMENT"
        elif db_type == "mysql" or "mysql+mysqlconnector":
            column_definition = "id INTEGER PRIMARY KEY AUTO_INCREMENT"
        elif db_type == "postgresql":
            column_definition = "id SERIAL PRIMARY KEY"
        else:
            raise ValueError(f"Unsupported database type: {db_type}")
        create_table_query = f"""
            CREATE TABLE admins (
                 {column_definition},
                username VARCHAR(50) NOT NULL UNIQUE,
                password VARCHAR(150) NOT NULL,
                active INTEGER NOT NULL DEFAULT 1,
                created_at TIMESTAMP DEFAULT CURRENT_TIMESTAMP
            )
            """
        return connection.query(query=create_table_query)
    return True


if __name__ == "__main__":
    app()



from lila.core.session import Session
from lila.core.responses import RedirectResponse, JSONResponse
from lila.core.request import Request
from functools import wraps
from app.helpers.helpers import get_token,generate_token


def login_required(func, key: str = "auth", url_return="/login"):
    @wraps(func)
    async def wrapper(request, *args, **kwargs):
        session_data = await check_session(request=request, key=key,return_JsonResponse=False)
        if not session_data:
            return RedirectResponse(url=url_return)
        return await func(request, *args, **kwargs)

    return wrapper


def session_active(func, key: str = "auth", url_return: str = "/dashboard"):
    @wraps(func)
    async def wrapper(request, *args, **kwargs):
        session_data = await check_session(request=request, key=key,return_JsonResponse=False)
        if session_data:
            return RedirectResponse(url=url_return)
        return await func(request, *args, **kwargs)

    return wrapper


async def check_session(request: Request, key: str='auth', return_JsonResponse: bool = True):
    session_data = Session.unsign(key=key, request=request)
    if return_JsonResponse:
        return JSONResponse({"session": False, "success": False}, status_code=401)
    return session_data


def validate_token(func):
    @wraps(func)
    async def wrapper(request: Request, *args, **kwargs):
        response = await check_token(request=request)
        if isinstance(response, JSONResponse):
            return response
        return await func(request, *args, **kwargs)

    return wrapper


async def check_token(request: Request):
    token = request.headers.get("Authorization")
    if not token:
        return JSONResponse(
            {"session": False, "message": "Invalid token"}, status_code=401
        )
    
    token = get_token(token=token)
    if isinstance(token, JSONResponse):
        return token
    return True



from lila.core.request import Request  # English: Handles HTTP requests in the application. | Español: Maneja solicitudes HTTP en la aplicación.
from lila.core.responses import  JSONResponse # English: Simplifies sending JSON responses. | Español: Simplifica el envío de respuestas JSON.
from lila.core.routing import Router # English: Manages routing for API endpoints. | Español: Administra las rutas para los puntos finales de la API.
from pydantic import  EmailStr,BaseModel  # English: Validates and parses data models for input validation. | Español: Valida y analiza modelos de datos para la validación de entradas.
from app.helpers.helpers import get_user_by_token
from app.middlewares.middlewares import validate_token, check_token, check_session

# English: Initialize the router instance for managing API routes.
# Español: Inicializa la instancia del enrutador para manejar rutas de la API.
router = Router()

# English: Define a simple API route that supports GET method.
# Español: Define una ruta de API simple que soporta el método GET.
@router.route(path="/api", methods=["GET"])
async def api(request: Request):
    """Api function"""  # use doc for descripction http://127.0.0.1:8000/openapi.json and http://127.0.0.1:8000/docs
    # English: Returns a simple JSON response for API verification. | Español: Devuelve una respuesta JSON simple para la verificación de la API.
    return JSONResponse({"api": True})


# English: Define an API route that supports GET and POST methods.
# Español: Define una ruta de API que soporta los métodos GET y POST.
@router.route(path="/api/token", methods=["GET", "POST"])
# English: Middleware to validate the JWT Token.
# Español: Middleware para validar token de JWT.
@validate_token
async def api_token(request: Request):
    """Api Token function"""  # use doc for descripction http://127.0.0.1:8001/openapi.json and http://127.0.0.1:8001/docs
    print(get_user_by_token(request=request))
    return JSONResponse({"api": True})


# English: Example data model using Pydantic.
# Español: Ejemplo de modelo de datos usando Pydantic.
class ExampleModel(BaseModel):
    email: EmailStr  # English: Ensures the email is valid. | Español: Garantiza que el email sea válido.
    password: str  # English: A plain string for the password. | Español: Una cadena simple para la contraseña.


# English: Defines an "api/example" route, using ExampleModel for input validation, with automatic documentation passing the "model" parameter ("model=ExampleModel").
# Español: Define una ruta "api/example", utilizando ExampleModel para la validación de entradas,con documentación automatica pasandole el parametro "model"("model=ExampleModel")
@router.route(path="/api/example", methods=["POST"], model=ExampleModel)
async def login(request: Request):
    """Example function get request json form"""  # use doc for descripction http://127.0.0.1:8000/openapi.json and http://127.0.0.1:8000/docs
    # English: Asynchronously parse JSON body from the request.
    # Español: Analiza asíncronamente el cuerpo JSON de la solicitud.
    body = await request.json()
    try:
        # English: Validate input data against the ExampleModel.
        # Español: Valida los datos de entrada contra ExampleModel.
        input = ExampleModel(**body)
    except Exception as e:
        return JSONResponse(
            {"success": False, "msg": f"Invalid JSON Body: {e}"}, status_code=400
        )

    email = input.email
    password = input.password
    response = JSONResponse({"email": email, "password": password})
    return response


# Example generate API REST CRUD with 'rest_crud_generate'
from app.connections import connection
from app.models.user import User
from pydantic import BaseModel, EmailStr


class UserModel(BaseModel):
    email: EmailStr
    name: str
    token: str
    password: str


# Example execute middlewares for rest crud generate
middlewares_user = {
    "get": [],
    "post": [],  
    "get_id": [],
    "put": [],
    "delete": [check_session],# Example of passing middlewares to the function  'rest_crud_generate'
}
# Rest crud generate ,base models SQl and models pydantic
router.rest_crud_generate(
    router=router,
    connection=connection,
    model_sql=User,
    model_pydantic=UserModel,
    select=["name", "email", "id", "created_at", "active"],
    delete_logic=True,
    active=True,
    middlewares=middlewares_user,
)


# English: Enable Swagger UI for API documentation.
# Español: Habilita Swagger UI para la documentación de la API.
router.swagger_ui()

# English: Generate OpenAPI JSON for external tools.
# Español: Genera JSON de OpenAPI para herramientas externas.
router.openapi_json()

# English: Retrieve all defined routes in the application.
# Español: Obtiene todas las rutas definidas en la aplicación.
routes = router.get_routes()



# Import necessary modules and functions from the core and middlewares
from lila.core.request import Request  # English: Importing Request class to handle HTTP requests | Español: Importando la clase Request para manejar las solicitudes HTTP
from lila.core.routing import Router # English: Importing Router class to define and manage routes | Español: Importando la clase Router para definir y manejar rutas
from lila.core.templates import render,renderMarkdown# English: Importing functions for rendering templates and Markdown files | Español: Importando funciones para renderizar plantillas y archivos Markdown
from lila.core.session import Session # English: Importing Session class for session handling | Español: Importando la clase Session para el manejo de sesiones
from lila.core.responses import RedirectResponse# English: Importing RedirectResponse to handle HTTP redirects | Español: Importando RedirectResponse para manejar redirecciones HTTP
from app.config import LANG_DEFAULT # English: Importing the default language setting from the environment | Español: Importando la configuración de idioma por defecto desde el entorno

# English: Creating an instance of Router to define routes
# Español: Creando una instancia del Router para definir las rutas
router = Router()

#marker_react

# English: Mounting statics files ,in folder 'static',url ='/public'
#  Español: Montando los archivos estaticos en la carpeta 'static',url ='/public'
router.mount()

# English: Example render html file with Jinja2, passing translation parameters in the context
# Español : Ejemplo renderizar archivo html con Jinja2, pasandole parametros de traduccion en el contexto
@router.route(path="/", methods=["GET"])
async def home(request: Request):
    response = render(
        request=request, template="index"
    )  # English: Renders the 'index' template with  translations | Español: Renderiza la plantilla 'index' con traducciones
    return response


# English: Example default lang in the path
# Español : Ejemplo de dejar el idioma por defecto en la ruta
@router.route(path="/es", methods=["GET"])
async def home(request: Request):
    response = render(
        request=request, template="index",lang_default="es"
    )  
    return response

# English: Example default lang in the path
# Español : Ejemplo de dejar el idioma por defecto en la ruta
@router.route(path="/en", methods=["GET"])
async def home(request: Request):
    response = render(
        request=request, template="index",lang_default="en"
    )  
    return response

# English : Example for render 'markdown' file
# Español : Ejemplo para renderizar un archivo 'markdown'
@router.route(path="/markdown", methods=["GET"])
async def home(request: Request):
    # English: Define a list of CSS files to include in the response
    # Español: Define una lista de archivos CSS para incluir en la respuesta
    css = ["/public/css/styles.css"]
    # English: Renders a markdown file with materialcss styling
    # Español: Renderiza un archivo markdown con el estilo materialcss
    response = renderMarkdown(
        request=request, file="example", css_files=css
    )
    return response


# English: Example route for changing the language
# Español: Ejemplo de ruta para cambiar el idioma
@router.route(path="/set-language/{lang}", methods=["GET"])
async def set_language(request: Request):
    # English: Gets the language from the path parameters, falling back to the default if not found
    # Español: Obtiene el idioma de los parámetros de la ruta, usando el valor por defecto si no se encuentra
    lang = request.path_params.get("lang", LANG_DEFAULT)
    if not lang:
        lang = request.query_params.get("lang", LANG_DEFAULT)
    referer = request.headers.get("Referer", "/")
    response = RedirectResponse(url=referer)
    Session.setSession(name_cookie="lang", new_val=lang, response=response)
    return response


 

# English: Get all the defined routes
#  Español: Obtiene todas las rutas definidas
routes = router.get_routes()
